@@ -35,7 +35,9 @@
 #include <libgnomeui/libgnomeui.h>
 #include "eggtrayicon.h"
 #include "eel-ellipsizing-label.h"
+#include "gossip-vcard-dialog.h"
 #include "gossip-add-contact.h"
+#include "gossip-contact.h"
 #include "gossip-jid.h"
 #include "gossip-marshal.h"
 #include "gossip-utils.h"
@@ -48,6 +50,7 @@
 #include "gossip-idle.h"
 #include "gossip-startup-druid.h"
 #include "gossip-preferences.h"
+#include "gossip-presence.h"
 #include "gossip-account-dialog.h"
 #include "gossip-stock.h"
 #include "gossip-roster-view.h"
@@ -87,54 +90,54 @@
 extern GConfClient *gconf_client;
 
 struct _GossipAppPriv {
-	LmConnection        *connection;
-	GtkWidget           *window;
+	LmConnection     *connection;
+	GtkWidget        *window;
 
-	EggTrayIcon         *tray_icon;
-	GtkWidget           *tray_event_box;
-	GtkWidget           *tray_image;
-	GtkTooltips         *tray_tooltips;
-	GList               *tray_flash_icons;
-	guint                tray_flash_timeout_id;
+	EggTrayIcon      *tray_icon;
+	GtkWidget        *tray_event_box;
+	GtkWidget        *tray_image;
+	GtkTooltips      *tray_tooltips;
+	GList            *tray_flash_icons;
+	guint             tray_flash_timeout_id;
 
-	GtkWidget           *popup_menu;
-	GtkWidget           *popup_menu_status_item;
-	GtkWidget           *show_popup_item;
-	GtkWidget           *hide_popup_item;
+	GtkWidget        *popup_menu;
+	GtkWidget        *popup_menu_status_item;
+	GtkWidget        *show_popup_item;
+	GtkWidget        *hide_popup_item;
 	
-	GossipRoster        *roster;
-	GossipRosterView    *roster_view;
+	GossipRoster     *roster;
+	GossipRosterView *roster_view;
 
-	GossipAccount       *account;
-	gchar               *overridden_resource;
+	GossipAccount    *account;
+	gchar            *overridden_resource;
 
-	GossipJID           *jid;
+	GossipJID        *jid;
 
 	/* Widgets that are enabled when we're connected/disconnected. */
-	GList               *enabled_connected_widgets;
-	GList               *enabled_disconnected_widgets;
+	GList            *enabled_connected_widgets;
+	GList            *enabled_disconnected_widgets;
 
 	/* Status popup. */
-	GtkWidget           *status_button_hbox;
-	GtkWidget           *status_button;
-	GtkWidget           *status_popup;
-	GtkWidget           *status_label;
-	GtkWidget           *status_image;
+	GtkWidget        *status_button_hbox;
+	GtkWidget        *status_button;
+	GtkWidget        *status_popup;
+	GtkWidget        *status_label;
+	GtkWidget        *status_image;
 
-	guint                status_flash_timeout_id;
-	time_t               leave_time;
+	guint             status_flash_timeout_id;
+	time_t            leave_time;
 
 	/* Current status/overridden away message. */
-	gchar               *status_text;
-	gchar               *away_message;
+	gchar            *status_text;
+	gchar            *away_message;
 
 	/* Set by the user (available/busy). */
-	GossipShow           explicit_show;
+	GossipShow        explicit_show;
 
 	/* Autostatus (away/xa), overrides explicit_show. */
-	GossipShow           auto_show;     
-
-	guint                size_timeout_id;
+	GossipShow        auto_show;     
+	
+	guint             size_timeout_id;
 };
 
 typedef struct {
@@ -162,8 +165,8 @@ static void     app_init                             (GossipApp          *app);
 static void     app_finalize                         (GObject            *object);
 static void     app_main_window_destroy_cb           (GtkWidget          *window,
 						      GossipApp          *app);
-static void     app_item_activated_cb                (GossipRosterView   *roster_view,
-						      GossipRosterItem   *item,
+static void     app_contact_activated_cb             (GossipRosterView   *roster_view,
+						      GossipContact      *contact,
 						      GossipApp          *app);
 static gboolean app_idle_check_cb                    (GossipApp          *app);
 static void     app_quit_cb                          (GtkWidget          *window,
@@ -182,6 +185,8 @@ static void     app_show_offline_key_changed_cb      (GConfClient        *client
 						      gpointer            check_menu_item);
 static void     app_preferences_cb                   (GtkWidget          *widget,
 						      GossipApp          *app);
+static void     app_personal_details_cb              (GtkWidget          *widget,
+						      GossipApp          *app);
 static void     app_account_information_cb           (GtkWidget          *widget,
 						      GossipApp          *app);
 static void     app_status_messages_cb               (GtkWidget          *widget,
@@ -249,10 +254,9 @@ static gboolean app_complete_name_key_press_event_cb  (GtkEntry           *entry
 						      CompleteNameData    *data);
 static void     app_complete_name_activate_cb         (GtkEntry           *entry,
 						      CompleteNameData    *data);
-static gchar *  app_complete_item_to_string          (gpointer            data);
 static void     app_toggle_visibility                (void);
 static void     app_tray_push_message                (LmMessage          *m);
-static gboolean app_tray_pop_message                 (GossipRosterItem   *item);
+static gboolean app_tray_pop_message                 (GossipContact      *contact);
 static void     app_tray_update_tooltip              (void);
 static GtkWidget *
 app_create_status_menu                               (gboolean            from_window);
@@ -403,6 +407,7 @@ app_init (GossipApp *singleton_app)
 			      "actions_add_contact", "activate", app_add_contact_cb,
 			      "actions_show_offline", "toggled", app_show_offline_cb,
 			      "edit_preferences", "activate", app_preferences_cb,
+			      "edit_personal_details", "activate", app_personal_details_cb,
 			      "edit_account_information", "activate", app_account_information_cb,
 			      "edit_status_messages", "activate", app_status_messages_cb,
 			      "help_about", "activate", app_about_cb,
@@ -456,8 +461,8 @@ app_init (GossipApp *singleton_app)
 			   GTK_WIDGET (priv->roster_view));
 
 	g_signal_connect (priv->roster_view,
-			  "item_activated",
-			  G_CALLBACK (app_item_activated_cb),
+			  "contact_activated",
+			  G_CALLBACK (app_contact_activated_cb),
 			  app);
 
 	app_tray_create_menu ();
@@ -651,20 +656,29 @@ app_authentication_cb (LmConnection *connection,
 	app_update_show ();
 }
 
-static gint
-app_name_complete_cmp_func (const gchar *s1, const gchar *s2, gsize n)
+static void
+app_new_message_presence_data_func (GtkCellLayout   *cell_layout,
+				    GtkCellRenderer *cell,
+				    GtkTreeModel    *tree_model,
+				    GtkTreeIter     *iter,
+				    gpointer         unused)
 {
-	gchar *u1, *u2;
-	gint   ret_val;
+	GossipContact *c;
+	GdkPixbuf     *pixbuf;
 	
-	u1 = g_utf8_casefold (s1, n);
-	u2 = g_utf8_casefold (s2, n);
+	gtk_tree_model_get (tree_model, iter, 0, &c, -1);
 
-	ret_val = g_utf8_collate (u1, u2);
-	g_free (u1);
-	g_free (u2);
+	if (!gossip_contact_is_online (c)) {
+		pixbuf = gossip_utils_get_pixbuf_offline ();
+	} else {
+		GossipPresence *p;
 
-	return ret_val;
+		p = gossip_contact_get_presence (c);
+		pixbuf = gossip_utils_get_pixbuf_from_presence (p);
+	}
+
+	g_object_set (cell, "pixbuf", pixbuf, NULL);
+	g_object_unref (pixbuf);
 }
 
 static void
@@ -673,11 +687,14 @@ app_new_message (gboolean use_roster_selection, gboolean be_transient)
 	GossipAppPriv    *priv;
 	const gchar      *selected_name = NULL;
 	GList            *l;
-	GList            *items;
-	GossipRosterItem *item = NULL;
+	GList            *contacts = NULL;
+	GossipContact    *contact = NULL;
 	GtkWidget        *frame;
-	CompleteNameData  *data;
-	GtkWindow         *parent;
+	CompleteNameData *data;
+	GtkWindow        *parent;
+	GList            *items;
+	GtkListStore     *model;
+	GtkCellRenderer  *cell;
 	
 	priv = app->priv;
 
@@ -712,14 +729,25 @@ app_new_message (gboolean use_roster_selection, gboolean be_transient)
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (data->dialog)->vbox),
 			    frame, TRUE, TRUE, 8);
 	gtk_widget_show (frame);
+
+	model = gtk_list_store_new (2, G_TYPE_POINTER, G_TYPE_STRING);
+	data->combo = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL (model), 1);
+
+	cell = gtk_cell_renderer_pixbuf_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (data->combo), cell, FALSE);
+	/*gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (data->combo), cell,
+				       "pixbuf", 0); */
+	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (data->combo),
+					    cell, 
+					    app_new_message_presence_data_func,
+					    NULL, NULL);
+					    
+	gtk_cell_layout_reorder (GTK_CELL_LAYOUT (data->combo), cell, 0);
 	
-	data->combo = gtk_combo_new ();
-	gtk_combo_set_use_arrows (GTK_COMBO (data->combo), FALSE);
-	gtk_combo_disable_activate (GTK_COMBO (data->combo));
 	gtk_container_add (GTK_CONTAINER (frame), data->combo);
 	gtk_widget_show (data->combo);
 
-	data->entry = GTK_COMBO (data->combo)->entry;
+	data->entry = GTK_BIN (data->combo)->child;
 
 	g_signal_connect_after (data->entry,
 				"insert_text",
@@ -735,45 +763,74 @@ app_new_message (gboolean use_roster_selection, gboolean be_transient)
 			  "activate",
 			  G_CALLBACK (app_complete_name_activate_cb),
 			  data);
-
 	g_signal_connect (data->dialog,
 			  "response",
 			  G_CALLBACK (app_complete_name_response_cb),
 			  data);
 	
-	data->completion = g_completion_new (app_complete_item_to_string);
+	data->completion = g_completion_new ((GCompletionFunc) gossip_contact_get_name);
 	g_completion_set_compare (data->completion, 
-				  app_name_complete_cmp_func);
-	if (use_roster_selection) {
-		item = gossip_roster_view_get_selected_item (priv->roster_view);
-	} else {
-		item = NULL;
-	}
+				  gossip_utils_str_n_case_cmp);
 
-	data->names = NULL;
+	if (use_roster_selection) {
+		GossipRosterItem *item;
+		
+		item = gossip_roster_view_get_selected_item (priv->roster_view);
+		if (item) {
+			contact = gossip_roster_get_contact_from_item (priv->roster, item);
+		}
+	} 
 
 	items = gossip_roster_get_all_items (priv->roster);
 	for (l = items; l; l = l->next) {
-		GossipRosterItem *roster_item = (GossipRosterItem *) l->data;
-		
-		data->names = g_list_prepend (data->names, 
-					      g_strdup (gossip_roster_item_get_name (roster_item)));
+		GossipRosterItem *roster_item;
+		GossipContact    *c;
 
-		if (item == roster_item) {
+		roster_item = (GossipRosterItem *) l->data;
+		c = gossip_roster_get_contact_from_item (priv->roster,
+							 roster_item);
+
+		/*data->names = g_list_prepend (data->names, 
+					      g_strdup (gossip_contact_get_name (c)));
+*/
+		if (contact && gossip_contact_equal (contact, c)) {
 			/* Got the selected one, select it in the combo. */
-			selected_name = gossip_roster_item_get_name (roster_item);
+			selected_name = gossip_contact_get_name (c);
 		}
+
+		contacts = g_list_prepend (contacts, 
+					   gossip_contact_ref (c));
 	}
-	
+
+	contacts = g_list_sort (contacts, gossip_contact_name_case_compare);
+	for (l = contacts; l; l = l->next) {
+		GtkTreeIter     iter;
+		GossipContact  *c;
+		
+		c = GOSSIP_CONTACT (l->data);
+
+		gtk_list_store_append (model, &iter);
+
+		gtk_list_store_set (model, &iter,
+				    0, c,
+				    1, gossip_contact_get_name (c),
+				    -1);
+	}
+#if 0	
+	data->names = g_list_sort (data->names, 
+				   (GCompareFunc) gossip_utils_str_case_cmp);
 	if (data->names) {
 		gtk_combo_set_popdown_strings (GTK_COMBO (data->combo),
 					       data->names);
 	}
-
-	if (items) {
-		g_completion_add_items (data->completion, items);
+#endif
+	/* contacts = g_list_sort (contacts,
+				gossip_contact_name_case_compare);*/
+	if (contacts) {
+		g_completion_add_items (data->completion, contacts);
 	}
 
+#if 0
 	if (selected_name) {
 		gtk_entry_set_text (GTK_ENTRY (data->entry), selected_name);
 		gtk_editable_select_region (GTK_EDITABLE (data->entry), 0, -1);
@@ -782,7 +839,7 @@ app_new_message (gboolean use_roster_selection, gboolean be_transient)
 	}
 
 	gtk_widget_grab_focus (data->entry);
-	
+#endif	
 	gtk_widget_show (data->dialog);
 }
 
@@ -845,6 +902,12 @@ app_preferences_cb (GtkWidget *widget, GossipApp *app)
 	gossip_preferences_show ();
 }
 
+static void
+app_personal_details_cb (GtkWidget *widget, GossipApp *app)
+{
+	gossip_vcard_dialog_show ();
+}
+
 static void
 app_account_information_cb (GtkWidget *widget, GossipApp *app)
 {
@@ -951,7 +1014,7 @@ app_handle_subscription_request (GossipApp *app, LmMessage *m)
 	from = lm_message_node_get_attribute (m->node, "from");
 
 
-	tmp = g_strdup_printf (_("%s wants to be notified of your presence."), from);
+	tmp = g_strdup_printf (_("%s wants to be notified of your status."), from);
 	str = g_strdup_printf ("<span weight='bold' size='larger'>%s</span>", tmp);
 	g_free (tmp);
 
@@ -971,6 +1034,7 @@ app_handle_subscription_request (GossipApp *app, LmMessage *m)
 			   "add_check_button", add_check_button);
 
 	jid = gossip_jid_new (from);
+
 	if (gossip_roster_get_item (app->priv->roster, jid)) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (add_check_button),
 					      FALSE);
@@ -1208,16 +1272,16 @@ app_client_disconnected_cb (LmConnection       *connection,
 }
 
 static void
-app_item_activated_cb (GossipRosterView *roster,
-		       GossipRosterItem *item,
-		       GossipApp        *app)
+app_contact_activated_cb (GossipRosterView *roster,
+			  GossipContact    *contact,
+			  GossipApp        *app)
 {
 	GossipChat *chat;
 
-	chat = gossip_chat_get_for_item (item, TRUE);
+	chat = gossip_chat_get_for_contact (contact, TRUE);
 	gossip_chat_present (chat);
 
-	app_tray_pop_message (item);
+	app_tray_pop_message (contact);
 }
 
 void
@@ -1231,7 +1295,7 @@ gossip_app_join_group_chat (const gchar *room,
 	tmp = g_strdup_printf ("%s@%s", room, server);
 	jid = gossip_jid_new (tmp);
 	g_free (tmp);
-	gossip_group_chat_new (jid, nick);
+	gossip_group_chat_show (jid, nick);
 	gossip_jid_unref (jid);
 }
 
@@ -1461,7 +1525,8 @@ app_setup_conn_dependent_menu_items (GladeXML *glade)
 		"actions_join_group_chat",
 		"actions_send_chat_message",
 		"actions_add_contact",
-		"status_button"
+		"status_button",
+		"edit_personal_details"
 	};
 	
 	const gchar *disconnect_widgets[] = {
@@ -1516,23 +1581,31 @@ app_complete_name_response_cb (GtkWidget        *dialog,
 	GList       *l;
 
 	if (response == GTK_RESPONSE_OK) {
-		GossipRosterItem *item = NULL;
-		str = gtk_entry_get_text (GTK_ENTRY (data->entry));
+		GossipRosterItem *item;
+		GossipContact    *contact;
 		
-		item = gossip_roster_find_item (gossip_app_get_roster (), str);
-		if (!item) {
-			/* FIXME: Display error dialog... */
-			g_warning ("'%s' is not a valid JID or nick name.", str);
-		}
+		str = gtk_entry_get_text (GTK_ENTRY (data->entry));
 
-		chat = gossip_chat_get_for_item (item, TRUE);
-				
-		gossip_chat_present (chat);
+		if (str && strcmp (str, "") != 0) {
+			item = gossip_roster_find_item (gossip_app_get_roster (),
+							str);
+			if (!item) {
+				/* FIXME: Display error dialog... */
+				g_warning ("'%s' is not a valid JID or nick name.", str);
+			}
+
+			contact = gossip_roster_get_contact_from_item (gossip_app_get_roster (),
+							       item);
+			chat = gossip_chat_get_for_contact (contact, TRUE);
+			
+			gossip_chat_present (chat);
+		}
 	}
 
 	for (l = data->names; l; l = l->next) {
 		g_free (l->data);
 	}
+
 	g_list_free (data->names);
 	g_free (data);
 	
@@ -1618,14 +1691,6 @@ app_complete_name_activate_cb (GtkEntry        *entry,
 	gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK);
 }
 
-static gchar *
-app_complete_item_to_string (gpointer data)
-{ 
-	GossipRosterItem *item = data;
-
-	return (gchar *) gossip_roster_item_get_name (item);
-}
-
 GossipShow 
 gossip_app_get_show (void)
 {
@@ -1862,6 +1927,7 @@ app_tray_push_message (LmMessage *m)
 	const gchar      *from;
 	GossipJID        *jid; 
 	GossipRosterItem *item;
+	GossipContact    *contact;
 	GList            *l;
 		
 	priv = app->priv;
@@ -1871,32 +1937,32 @@ app_tray_push_message (LmMessage *m)
 
 	item = gossip_roster_get_item (priv->roster, jid);
 	if (!item) {
-		item = gossip_roster_item_new (jid);
+		contact = gossip_contact_new (GOSSIP_CONTACT_TYPE_TEMPORARY);
+		gossip_contact_set_jid (contact, jid);
 	} else {
-		gossip_roster_item_ref (item);
+		contact = gossip_roster_get_contact_from_item (priv->roster,
+							       item);
+		gossip_contact_ref (contact);
 	}
 
 	gossip_jid_unref (jid);
 	
-	l = g_list_find_custom (priv->tray_flash_icons, item,
-				(GCompareFunc) gossip_roster_item_compare);
+	l = g_list_find_custom (priv->tray_flash_icons, contact,
+				(GCompareFunc) gossip_contact_compare);
 	if (l) {
 		return;
 	}
 
-	priv->tray_flash_icons = g_list_append (priv->tray_flash_icons, 
-						gossip_roster_item_ref (item));
-
+	priv->tray_flash_icons = g_list_append (priv->tray_flash_icons, contact);
 
 	app_tray_flash_start ();
 	app_tray_update_tooltip ();
 
-	gossip_roster_view_flash_item (priv->roster_view, item, TRUE);
-	gossip_roster_item_unref (item);
+	gossip_roster_view_flash_contact (priv->roster_view, contact, TRUE);
 }
 
 static gboolean
-app_tray_pop_message (GossipRosterItem *item)
+app_tray_pop_message (GossipContact *contact)
 {
 	GossipAppPriv *priv;
 	GossipChat    *chat;
@@ -1908,19 +1974,19 @@ app_tray_pop_message (GossipRosterItem *item)
 		return FALSE;
 	}
 
-	if (!item) {
-		item = priv->tray_flash_icons->data;
+	if (!contact) {
+		contact = priv->tray_flash_icons->data;
 	}
 	
-	chat = gossip_chat_get_for_item (item, TRUE);
+	chat = gossip_chat_get_for_contact (contact, TRUE);
 	if (!chat) {
 		return FALSE;
 	}
 
 	gossip_chat_present (chat);
 
-	l = g_list_find_custom (priv->tray_flash_icons, item,
-				(GCompareFunc) gossip_roster_item_compare);
+	l = g_list_find_custom (priv->tray_flash_icons, contact,
+				(GCompareFunc) gossip_contact_compare);
 
 	if (!l) {
 		return FALSE;
@@ -1931,11 +1997,12 @@ app_tray_pop_message (GossipRosterItem *item)
 	app_tray_flash_maybe_stop ();
 	app_tray_update_tooltip ();
 
-	if (item) {
-		gossip_roster_view_flash_item (priv->roster_view, item, FALSE);
+	if (contact) {
+		gossip_roster_view_flash_contact (priv->roster_view, 
+						  contact, FALSE);
 	}
 	
-	gossip_roster_item_unref (item);
+	gossip_contact_unref (contact);
 
 	return TRUE;
 }
@@ -1946,7 +2013,7 @@ app_tray_update_tooltip (void)
 	GossipAppPriv    *priv;
 	const gchar      *name;
 	gchar            *str;
-	GossipRosterItem *item;
+	GossipContact    *contact;
 
 	priv = app->priv;
 
@@ -1957,9 +2024,9 @@ app_tray_update_tooltip (void)
 		return;
 	}
 
-	item = priv->tray_flash_icons->data;
-	
-	name = gossip_roster_item_get_name (item);
+	contact = priv->tray_flash_icons->data;
+
+	name = gossip_contact_get_name (contact);
 	
 	str = g_strdup_printf (_("New message from %s"), name);
 	
@@ -2068,14 +2135,14 @@ app_update_show (void)
 		if (status_text) {
 			status_label = g_strdup (priv->status_text);
 		} else {
-			status_label = g_strdup (gossip_utils_get_default_status (effective_show));
+			status_label = g_strdup (gossip_utils_get_default_status_show (effective_show));
 		}
 	}
 
 	if (status_text && status_text[0]) {
 		status_label = status_text;
 	} else {
-		status_label = gossip_utils_get_default_status (effective_show);
+		status_label = gossip_utils_get_default_status_show (effective_show);
 	}
 
 	eel_ellipsizing_label_set_text (EEL_ELLIPSIZING_LABEL (priv->status_label),
@@ -2407,7 +2474,7 @@ app_status_show_status_dialog (GossipShow   show,
 		
 	str = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	if (strcmp (str, gossip_utils_get_default_status (show)) == 0) {
+	if (strcmp (str, gossip_utils_get_default_status_show (show)) == 0) {
 		str = NULL;
 	}
 
