@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003-2004 Imendio HB
  * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <gvdbogaerde@pandora.be>
  *
  * This program is free software; you can redistribute it and/or
@@ -68,6 +68,10 @@ static void chat_window_conv_activate_cb      (GtkWidget             *menuitem,
 					       GossipChatWindow      *window);
 static void chat_window_close_activate_cb     (GtkWidget	     *menuitem,
 					       GossipChatWindow      *window);
+static void chat_window_copy_activate_cb      (GtkWidget             *menuitem,
+					       GossipChatWindow      *window);
+static void chat_window_paste_activate_cb     (GtkWidget             *menuitem,
+					       GossipChatWindow      *window);
 static void chat_window_tab_left_activate_cb  (GtkWidget	     *menuitem,
 					       GossipChatWindow      *window);
 static void chat_window_tab_right_activate_cb (GtkWidget	     *menuitem,
@@ -78,7 +82,7 @@ static gboolean chat_window_delete_event_cb   (GtkWidget	     *dialog,
 					       GdkEvent		     *event,
 					       GossipChatWindow	     *window);
 static void chat_window_presence_updated_cb   (gpointer               not_used,
-					       GossipRosterItem      *item,
+					       GossipContact         *contact,
 					       GossipChat	     *chat);
 static void chat_window_update_tooltip        (GossipChatWindow      *window,
 					       GossipChat            *chat);
@@ -116,6 +120,14 @@ chat_window_focus_in_event_cb                 (GtkWidget	     *widget,
 static const gchar *
 chat_window_get_name                          (GossipChatWindow      *window, 
 					       GossipChat            *chat);
+static void    chat_window_drag_data_received (GtkWidget             *widget,
+					       GdkDragContext        *context,
+					       int                    x,
+					       int                    y, 
+					       GtkSelectionData      *selection,
+					       guint                  info, 
+					       guint                  time, 
+					       GossipChatWindow      *window);
 
 /* Called from Glade, so it shouldn't be static */
 GtkWidget * chat_window_create_notebook (gpointer data);
@@ -137,6 +149,8 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *m_conv_log;
 	GtkWidget   *m_conv_info;
 	GtkWidget   *m_conv_close;
+	GtkWidget   *m_edit_copy;
+	GtkWidget   *m_edit_paste;
 	GtkWidget   *m_tabs_next;
 	GtkWidget   *m_tabs_prev;
 	GtkWidget   *m_tabs_left;
@@ -152,6 +166,14 @@ static guint tab_accel_keys[] = {
 	GDK_6, GDK_7, GDK_8, GDK_9, GDK_0
 };
 
+enum DndDropType {
+	DND_DROP_TYPE_JID,
+	NUM_DROP_TYPES
+};
+
+static GtkTargetEntry drop_types[] = {
+	{ "text/jid", 0, DND_DROP_TYPE_JID },
+};
 
 GType
 gossip_chat_window_get_type (void)
@@ -211,6 +233,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_conv_info", &priv->m_conv_info,
 				       "menu_conv_log", &priv->m_conv_log,
 				       "menu_conv_close", &priv->m_conv_close,
+				       "menu_edit_copy", &priv->m_edit_copy,
+				       "menu_edit_paste", &priv->m_edit_paste,
 				       "menu_tabs_next", &priv->m_tabs_next,
 				       "menu_tabs_prev", &priv->m_tabs_prev,
 				       "menu_tabs_left", &priv->m_tabs_left,
@@ -255,6 +279,14 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  "activate",
 			  G_CALLBACK (chat_window_close_activate_cb),
 			  window);
+	g_signal_connect (priv->m_edit_copy,
+			  "activate",
+			  G_CALLBACK (chat_window_copy_activate_cb),
+			  window);
+	g_signal_connect (priv->m_edit_paste,
+			  "activate",
+			  G_CALLBACK (chat_window_paste_activate_cb),
+			  window);
 	g_signal_connect_swapped (priv->m_tabs_prev,
 			          "activate",
 			          G_CALLBACK (gtk_notebook_prev_page),
@@ -308,6 +340,14 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  G_CALLBACK (chat_window_disconnected_cb),
 			  window);
 
+	gtk_drag_dest_set (GTK_WIDGET (priv->notebook), GTK_DEST_DEFAULT_ALL,
+			   drop_types, NUM_DROP_TYPES,
+			   GDK_ACTION_COPY);
+
+	g_signal_connect (priv->notebook, "drag-data-received",
+			  G_CALLBACK (chat_window_drag_data_received),
+			  window);
+
 	chat_windows = g_list_prepend (chat_windows, window);
 
 	priv->chats = NULL;
@@ -349,16 +389,16 @@ chat_window_get_status_pixbuf (GossipChatWindow *window,
 		pixbuf = gossip_utils_get_pixbuf_offline ();
 	}
 	else {
-		GossipRosterItem *item;
+		GossipContact *contact;
 
-		item = gossip_chat_get_item (chat);
-		if (gossip_roster_item_is_offline (item)) {
-			pixbuf = gossip_utils_get_pixbuf_offline ();
+		contact = gossip_chat_get_contact (chat);
+		if (gossip_contact_is_online (contact)) {
+			GossipPresence *presence;
+
+			presence = gossip_contact_get_presence (contact);
+			pixbuf = gossip_utils_get_pixbuf_from_presence (presence);
 		} else {
-			GossipShow show;
-
-			show  = gossip_roster_item_get_show (item);
-			pixbuf = gossip_utils_get_pixbuf_from_show (show);
+			pixbuf = gossip_utils_get_pixbuf_offline ();
 		}
 	}
 
@@ -417,17 +457,22 @@ chat_window_create_label (GossipChatWindow *window,
 	GtkWidget            *status_img;
 	const gchar          *name;
 	gint	              w, h;
-	GtkWidget            *eventbox;
+	GtkWidget            *evbox_tooltip;
+	GtkWidget            *evbox_hbox;
 	
 	priv = window->priv;
 	
 	hbox = gtk_hbox_new (FALSE, 0);
 
-	eventbox = gtk_event_box_new ();
+	evbox_tooltip = gtk_event_box_new ();
+
+	/* Make the event box invisible. */
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (evbox_tooltip), FALSE);
 	
 	status_img = gtk_image_new ();
 	g_object_set_data (G_OBJECT (chat), "chat-window-status-img", status_img);
-	g_object_set_data (G_OBJECT (chat), "chat-window-tooltip-widget", eventbox);
+	g_object_set_data (G_OBJECT (chat), "chat-window-tooltip-widget",
+			   evbox_tooltip);
 
 	name = chat_window_get_name (window, chat);
 
@@ -450,10 +495,13 @@ chat_window_create_label (GossipChatWindow *window,
 	gtk_widget_set_size_request (close_button, w, h);
 	gtk_container_add (GTK_CONTAINER (close_button), close_img);
 
-	gtk_container_add (GTK_CONTAINER (eventbox), name_label);
+	evbox_hbox = gtk_hbox_new (FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (hbox), status_img, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), eventbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (evbox_hbox), status_img, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (evbox_hbox), name_label, TRUE, TRUE, 0);
+
+	gtk_container_add (GTK_CONTAINER (evbox_tooltip), evbox_hbox);
+	gtk_box_pack_start (GTK_BOX (hbox), evbox_tooltip, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
 	g_signal_connect (close_button,
@@ -484,6 +532,7 @@ chat_window_update_title (GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	gchar                *title; 
+	GdkPixbuf 	     *pixbuf;
 
 	priv = window->priv;
 
@@ -493,6 +542,12 @@ chat_window_update_title (GossipChatWindow *window)
 						       priv->current_chat));
 
 	gtk_window_set_title (GTK_WINDOW (window->priv->dialog), title);
+	if (window->priv->new_msg) {
+		pixbuf = gossip_utils_get_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE);
+		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), pixbuf);
+        } else {
+		gtk_window_set_icon (GTK_WINDOW (window->priv->dialog), NULL);
+	}
 	g_free (title);
 }
 
@@ -531,24 +586,24 @@ static void
 chat_window_log_activate_cb (GtkWidget *menuitem,GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv = window->priv;
-	GossipRosterItem     *item;
+	GossipContact        *contact;
 
-	item = gossip_chat_get_item (priv->current_chat);
+	contact = gossip_chat_get_contact (priv->current_chat);
 	
-	gossip_log_show (gossip_roster_item_get_jid (item));
+	gossip_log_show (gossip_contact_get_jid (contact));
 }
 
 static void
 chat_window_info_activate_cb (GtkWidget *menuitem, GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
-	GossipRosterItem     *item;
+	GossipContact        *contact;
 	
 	priv = window->priv;
 
-	item = gossip_chat_get_item (priv->current_chat);
+	contact = gossip_chat_get_contact (priv->current_chat);
 
-	gossip_contact_info_new (gossip_roster_item_get_jid (item),
+	gossip_contact_info_new (gossip_contact_get_jid (contact),
 				 chat_window_get_name (window, priv->current_chat));
 }
 
@@ -557,13 +612,13 @@ chat_window_conv_activate_cb (GtkWidget        *menuitem,
 			      GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
-	GossipRosterItem     *item;
+	GossipContact        *contact;
 	gboolean              log_exists;
 
 	priv = window->priv;
 	
-	item = gossip_chat_get_item (priv->current_chat);
-	log_exists = gossip_log_exists (gossip_roster_item_get_jid (item));
+	contact = gossip_chat_get_contact (priv->current_chat);
+	log_exists = gossip_log_exists (gossip_contact_get_jid (contact));
 	gtk_widget_set_sensitive (priv->m_conv_log, log_exists);
 }
 
@@ -576,6 +631,32 @@ chat_window_close_activate_cb (GtkWidget        *menuitem,
 	gossip_chat_window_remove_chat (window, window->priv->current_chat);
 }
 
+static void
+chat_window_copy_activate_cb (GtkWidget        *menuitem,
+			      GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	
+	g_return_if_fail (GOSSIP_IS_CHAT_WINDOW (window));
+
+	priv = window->priv;
+
+	gossip_chat_copy (priv->current_chat);
+}
+
+static void
+chat_window_paste_activate_cb (GtkWidget        *menuitem,
+			       GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	
+	g_return_if_fail (GOSSIP_IS_CHAT_WINDOW (window));
+	
+	priv = window->priv;
+
+	gossip_chat_paste (priv->current_chat);
+}
+
 static void
 chat_window_tab_left_activate_cb (GtkWidget        *menuitem,
 				  GossipChatWindow *window)
@@ -585,13 +666,13 @@ chat_window_tab_left_activate_cb (GtkWidget        *menuitem,
 	gint                  index;
 
 	priv = window->priv;
-	
+
 	chat = priv->current_chat;
 	index = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 	if (index <= 0) {
 		return;
 	}
-	
+
 	gossip_notebook_move_page (GOSSIP_NOTEBOOK (priv->notebook),
 				   GOSSIP_NOTEBOOK (priv->notebook),
 				   gossip_chat_get_widget (chat),
@@ -611,7 +692,7 @@ chat_window_tab_right_activate_cb (GtkWidget        *menuitem,
 	gint                  index;
 
 	priv = window->priv;
-	
+
 	chat = priv->current_chat;
 	index = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 
@@ -631,7 +712,7 @@ chat_window_detach_activate_cb (GtkWidget        *menuitem,
 {
 	GossipChatWindow *new_window;
 	GossipChat       *chat;
-	
+
 	chat = window->priv->current_chat;
 	new_window = gossip_chat_window_new ();
 
@@ -660,13 +741,13 @@ chat_window_delete_event_cb (GtkWidget        *dialog,
 
 static void
 chat_window_presence_updated_cb (gpointer           not_used,
-				 GossipRosterItem  *item,
+				 GossipContact     *contact,
 				 GossipChat        *chat)
 { 
 	GossipChatWindow *window;
-	
+
 	window = gossip_chat_get_window (chat);
-	
+
 	chat_window_update_status (window, chat);
 }
 
@@ -675,25 +756,28 @@ chat_window_update_tooltip (GossipChatWindow *window,
 			    GossipChat       *chat)
 {
 	GossipChatWindowPriv *priv;
-	GossipRosterItem     *item;
+	GossipContact        *contact;
 	GossipJID            *jid;
 	gchar                *str;
 	const gchar          *status;
 	GtkWidget            *widget;
-	
-	priv = window->priv;
-	
-	item = gossip_chat_get_item (chat);
-	jid = gossip_roster_item_get_jid (item);
+	GossipPresence       *presence;
 
-	status = gossip_roster_item_get_status (item);
+	priv = window->priv;
+
+	contact = gossip_chat_get_contact (chat);
+	jid = gossip_contact_get_jid (contact);
+
+	presence = gossip_contact_get_presence (contact);
+	status = gossip_presence_get_status (presence);
 
 	if (!status || strcmp (status, "") == 0) {
-		GossipShow show;
 
-		if (!gossip_roster_item_is_offline (item)) {
-			show = gossip_roster_item_get_show (item);
-			status = gossip_utils_get_default_status (show);
+		if (gossip_contact_is_online (contact)) {
+			GossipPresenceType p_type;
+
+			p_type = gossip_presence_get_type (presence);
+			status = gossip_utils_get_default_status (p_type);
 		} else {
 			status = _("Offline");
 		}
@@ -703,7 +787,16 @@ chat_window_update_tooltip (GossipChatWindow *window,
 			       gossip_jid_get_without_resource (jid),
 			       status);
 
-	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tooltip-widget");
+	if (g_list_find (priv->chats_composing, chat)) {
+		gchar *t_str;
+
+		t_str = str;
+		str = g_strconcat (t_str, "\n", _("Typing a message."), NULL);
+		g_free (t_str);
+	}
+
+	widget = g_object_get_data (G_OBJECT (chat), 
+				    "chat-window-tooltip-widget");
 
 	gtk_tooltips_set_tip (priv->tooltips,
 			      widget,
@@ -721,7 +814,7 @@ chat_window_name_changed_cb (GossipChat       *chat,
 	GtkLabel *label;
 
 	label = g_object_get_data (G_OBJECT (chat), "label");
-	
+
 	gtk_label_set_text (label, name);
 }
 
@@ -781,18 +874,21 @@ chat_window_switch_page_cb (GtkNotebook	     *notebook,
 			    gint	      page_num,
 			    GossipChatWindow *window)
 {
-	GossipChat *chat;
-	GtkWidget  *child;
+	GossipChatWindowPriv *priv;
+	GossipChat           *chat;
+	GtkWidget            *child;
 
+	priv = window->priv;
+	
 	child = gtk_notebook_get_nth_page (notebook, page_num); 	
 	chat = g_object_get_data (G_OBJECT (child), "chat");
 
-	if (window->priv->current_chat == chat) {
+	if (priv->current_chat == chat) {
 		return;
 	}
 
-	window->priv->current_chat = chat;
-	window->priv->chats_new_msg = g_list_remove (window->priv->chats_new_msg, chat);
+	priv->current_chat = chat;
+	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
 
 	chat_window_update_title (window);
 	chat_window_update_menu (window);
@@ -820,7 +916,7 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), child, label);
 
 	g_signal_connect (gossip_app_get_roster (),
-			  "item_presence_updated",
+			  "contact_presence_updated",
 			  G_CALLBACK (chat_window_presence_updated_cb),
 			  chat);
 	g_signal_connect (chat, "name_changed",
@@ -929,13 +1025,57 @@ static const gchar *
 chat_window_get_name (GossipChatWindow *window, GossipChat *chat)
 {
 	GossipChatWindowPriv *priv;
-	GossipRosterItem     *item;
+	GossipContact        *contact;
 
 	priv = window->priv;
 
-	item = gossip_chat_get_item (chat);
+	contact = gossip_chat_get_contact (chat);
 
-	return gossip_roster_item_get_name (item);
+	return gossip_contact_get_name (contact);
+}
+
+static void
+chat_window_drag_data_received (GtkWidget        *widget,
+				GdkDragContext   *context,
+				int               x,
+				int               y, 
+				GtkSelectionData *selection,
+				guint             info, 
+				guint             time, 
+				GossipChatWindow *window)
+{
+	GossipJID        *jid;
+	GossipRosterItem *item;
+	GossipContact    *contact;
+	GossipChat       *chat;
+	GossipChatWindow *old_window;
+	
+	jid = gossip_jid_new (selection->data);
+	item = gossip_roster_get_item (gossip_app_get_roster (), jid);
+	gossip_jid_unref (jid);
+	
+	if (!item) {
+		return;
+	}
+
+	contact = gossip_roster_get_contact_from_item (gossip_app_get_roster (),
+						       item);
+
+	chat = gossip_chat_get_for_contact (contact, TRUE);
+	old_window = gossip_chat_get_window (chat);
+	
+	if (old_window) {
+		if (old_window == window) {
+			goto finished;
+		}
+		
+		gossip_chat_window_remove_chat (old_window, chat);
+	}
+
+	gossip_chat_window_add_chat (window, chat);
+
+finished:
+	gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
 }
 
 GtkWidget *
