@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
- * Copyright (C) 2002 Richard Hult <richard@imendio.com>
+ * Copyright (C) 2002-2003 Imendio HB
  * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <gvdbogaerde@pandora.be>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,8 +25,9 @@
 #include <glib-object.h>
 #include <loudmouth/loudmouth.h>
 #include "gossip-app.h"
-#include "gossip-roster.h"
+#include "gossip-group-chat.h"
 #include "gossip-jid.h"
+#include "gossip-roster.h"
 
 G_BEGIN_DECLS
 
@@ -61,15 +61,16 @@ struct _GossipChatClass {
 };
 
 GType             gossip_chat_get_type           (void);
-GossipChat *      gossip_chat_get_for_item       (GossipRosterItem *item,
+GossipChat *      gossip_chat_get_for_contact    (GossipContact    *contact,
 						  gboolean          create);
-GossipChat *      gossip_chat_get_for_group_chat (GossipRosterItem *item);
+GossipChat *      gossip_chat_get_for_group_chat (GossipContact    *contact,
+						  GossipGroupChat  *g_chat);
 void              gossip_chat_append_message     (GossipChat       *chat,
 						  LmMessage        *message);
 void              gossip_chat_present            (GossipChat       *chat);
 LmHandlerResult   gossip_chat_handle_message     (LmMessage        *message);
 GtkWidget *       gossip_chat_get_widget         (GossipChat       *chat);
-GossipRosterItem *gossip_chat_get_item           (GossipChat       *chat);
+GossipContact *   gossip_chat_get_contact        (GossipChat       *chat);
 void              gossip_chat_set_window         (GossipChat       *chat,
 						  GossipChatWindow *window);
 GossipChatWindow *gossip_chat_get_window         (GossipChat       *chat);
@@ -77,6 +78,8 @@ void              gossip_chat_clear              (GossipChat       *chat);
 gchar *           gossip_chat_get_history        (GossipChat       *chat,
 						  gint              lines);
 void              gossip_chat_scroll_down        (GossipChat       *chat);
+void              gossip_chat_copy               (GossipChat       *chat);
+void              gossip_chat_paste              (GossipChat       *chat);
 
         
 G_END_DECLS
