@@ -40,15 +40,17 @@
 #define FLASH_TIMEOUT 500
 
 struct _GossipRosterViewPriv {
-	GtkTreeModel   *model;
-	GossipRoster   *roster;
+	GtkTreeModel        *model;
+	GossipRoster        *roster;
 
-	GtkItemFactory *item_popup_factory;
-	GtkItemFactory *group_popup_factory;
+	GtkItemFactory      *item_popup_factory;
+	GtkItemFactory      *group_popup_factory;
 
-	GHashTable     *flash_table;
+	GHashTable          *flash_table;
 
-	gboolean        show_offline;
+	gboolean             show_offline;
+
+	GtkTreeRowReference *drag_row;
 };
 
 typedef union {
@@ -61,6 +63,18 @@ typedef struct {
 	guint    flash_timeout_id;
 } FlashData;
 
+/* Drag & Drop types */
+enum DndDragType {
+	DND_DRAG_TYPE_JID,
+	NUM_DRAG_TYPES
+};
+
+static GtkTargetEntry drag_types[] = {
+	{ "text/jid", 0, DND_DRAG_TYPE_JID },
+};
+
+static GdkAtom drag_atoms[NUM_DRAG_TYPES];
+
 static void     roster_view_class_init               (GossipRosterViewClass *klass);
 static void     roster_view_init                     (GossipRosterView      *view);
 static void     roster_view_finalize                 (GObject               *object);
@@ -150,12 +164,21 @@ static void     roster_view_remove_item              (GossipRosterView      *vie
 static gboolean roster_view_iter_equal_item          (GtkTreeModel          *model,
 						      GtkTreeIter           *iter,
 						      GossipRosterItem      *item);
-
-
-
+static void     roster_view_drag_begin               (GtkWidget             *widget,
+						      GdkDragContext        *context,
+						      gpointer               user_data);
+static void     roster_view_drag_data_get            (GtkWidget             *widget,
+						      GdkDragContext        *contact,
+						      GtkSelectionData      *selection,
+						      guint                  info,
+						      guint                  time,
+						      gpointer               user_data);
+static void     roster_view_drag_end                 (GtkWidget             *widget,
+						      GdkDragContext        *context,
+						      gpointer               user_data);
 
 enum {
-	ITEM_ACTIVATED,
+	CONTACT_ACTIVATED,
 	LAST_SIGNAL
 };
 
@@ -291,8 +314,8 @@ roster_view_class_init (GossipRosterViewClass *klass)
 
 	object_class->finalize = roster_view_finalize;
 
-	signals[ITEM_ACTIVATED] = 
-		g_signal_new ("item_activated",
+	signals[CONTACT_ACTIVATED] = 
+		g_signal_new ("contact_activated",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
@@ -412,6 +435,7 @@ roster_view_setup_tree (GossipRosterView *view)
 	GossipRosterViewPriv *priv;
 	GtkCellRenderer      *cell;
 	GtkTreeViewColumn    *col;
+	static int            setup = 0;
 
 	priv = view->priv;
 	
@@ -437,7 +461,32 @@ roster_view_setup_tree (GossipRosterView *view)
 						 view,
 						 NULL);
 
+	if (!setup) {
+		int i;
+		
+		for (i = 0; i < NUM_DRAG_TYPES; ++i) {
+			drag_atoms[i] = gdk_atom_intern (drag_types[i].target,
+							 FALSE);
+		}
+
+		setup = 1;
+	}
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view), col);
+
+	gtk_drag_source_set (GTK_WIDGET (view), GDK_BUTTON1_MASK, 
+			     drag_types, NUM_DRAG_TYPES, 
+			     GDK_ACTION_COPY);
+
+	g_signal_connect (GTK_TREE_VIEW (view), "drag-begin",
+			  G_CALLBACK (roster_view_drag_begin),
+			  NULL);
+	g_signal_connect (GTK_TREE_VIEW (view), "drag-data-get",
+			  G_CALLBACK (roster_view_drag_data_get),
+			  NULL);
+	g_signal_connect (GTK_TREE_VIEW (view), "drag-end",
+			  G_CALLBACK (roster_view_drag_end),
+			  NULL);
 }
 
 static void
@@ -1076,6 +1125,106 @@ roster_view_iter_equal_item (GtkTreeModel     *model,
 	return FALSE;
 }
 
+static void
+roster_view_drag_begin (GtkWidget      *widget,
+			GdkDragContext *context,
+			gpointer        user_data)
+{
+	GossipRosterViewPriv *priv;
+	GtkTreeSelection     *selection;
+	GtkTreeModel         *model;
+	GtkTreePath          *path;
+	GtkTreeIter           iter;
+
+	priv = GOSSIP_ROSTER_VIEW (widget)->priv;
+	
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		return;
+	}
+
+	path = gtk_tree_model_get_path (model, &iter);
+	priv->drag_row = gtk_tree_row_reference_new (model, path);
+	gtk_tree_path_free (path);
+
+	/* FIXME: Set a nice icon */
+}
+
+static void
+roster_view_drag_data_get (GtkWidget             *widget,
+			   GdkDragContext        *contact,
+			   GtkSelectionData      *selection,
+			   guint                  info,
+			   guint                  time,
+			   gpointer               user_data)
+{
+	GossipRosterViewPriv *priv;
+	GtkTreePath          *src_path;
+	GtkTreeIter           iter;
+	GtkTreeModel         *model;
+	RosterElement        *e;
+	gboolean              is_group;
+	GossipJID            *jid;
+	const gchar          *str_jid;
+	
+	priv = GOSSIP_ROSTER_VIEW (widget)->priv;
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+	if (!priv->drag_row) {
+		return;
+	}
+
+	src_path = gtk_tree_row_reference_get_path (priv->drag_row);
+	if (!src_path) {
+		return;
+	}
+
+	if (!gtk_tree_model_get_iter (model, &iter, src_path)) {
+		gtk_tree_path_free (src_path);
+		return;
+	}
+
+	gtk_tree_path_free (src_path);
+
+	gtk_tree_model_get (model, &iter,
+			    COL_IS_GROUP, &is_group,
+			    COL_ELEMENT, &e,
+			    -1);
+	
+	if (is_group) {
+		return;
+	}
+
+	jid = gossip_roster_item_get_jid (e->item);
+	str_jid = gossip_jid_get_full (jid);
+	
+	switch (info) {
+	case DND_DRAG_TYPE_JID:
+		gtk_selection_data_set (selection, drag_atoms[info], 8, 
+					str_jid, strlen (str_jid) + 1);
+		break;
+	default:
+		return;
+	}
+	
+}
+
+static void
+roster_view_drag_end (GtkWidget      *widget,
+		      GdkDragContext *context,
+		      gpointer        user_data)
+{
+	GossipRosterViewPriv *priv;
+
+	priv = GOSSIP_ROSTER_VIEW (widget)->priv;
+
+	if (priv->drag_row) {
+		gtk_tree_row_reference_free (priv->drag_row);
+		priv->drag_row = NULL;
+	}
+}
+
+
 static gboolean
 roster_view_find_item (GossipRosterView  *view,
 		       GtkTreeIter       *iter,
@@ -1208,7 +1357,11 @@ roster_view_row_activated_cb (GossipRosterView  *view,
 	d(g_print ("Row activated!\n"));
 	
 	if (!is_group) {
-		g_signal_emit (view, signals[ITEM_ACTIVATED], 0, e->item);
+		GossipContact *contact;
+
+		contact = gossip_roster_get_contact_from_item (priv->roster,
+							       e->item);
+		g_signal_emit (view, signals[CONTACT_ACTIVATED], 0, contact);
 	}
 }
 
@@ -1768,18 +1921,26 @@ roster_view_flash_timeout_func (FlashTimeoutData *fdata)
 }
 
 void
-gossip_roster_view_flash_item (GossipRosterView *view,
-			       GossipRosterItem *item,
-			       gboolean          flash)
+gossip_roster_view_flash_contact (GossipRosterView *view,
+				  GossipContact    *contact,
+				  gboolean          flash)
 {
 	GossipRosterViewPriv *priv;
 	FlashData            *flash_data;
+	GossipRosterItem     *item;
 	
 	g_return_if_fail (GOSSIP_IS_ROSTER_VIEW (view));
-	g_return_if_fail (item != NULL);
+	g_return_if_fail (contact != NULL);
 
 	priv = view->priv;
 	
+	item = gossip_roster_get_item (priv->roster, 
+				       gossip_contact_get_jid (contact));
+
+	if (!item) {
+		item = gossip_roster_item_new (gossip_contact_get_jid (contact));
+	}
+	
 	flash_data = g_hash_table_lookup (priv->flash_table, item);
 
 	if (flash && !flash_data) {
