@@ -474,28 +474,6 @@ gossip_password_dialog_run (GossipAccount *account, GtkWindow *parent)
 	return password;
 }
 
-const gchar *
-gossip_get_icon_for_show_string (const gchar *str)
-{
-	if (!str || !str[0]) {
-		return GOSSIP_STOCK_AVAILABLE;
-	}
-	else if (strcmp (str, "dnd") == 0) {
-		return GOSSIP_STOCK_BUSY;
-	}
-	else if (strcmp (str, "away") == 0) {
-		return GOSSIP_STOCK_AWAY;
-	}
-	else if (strcmp (str, "xa") == 0) {
-		return GOSSIP_STOCK_EXT_AWAY;
-	}
-	/* We don't support chat, so treat it like available. */
-	else if (strcmp (str, "chat") == 0) {
-		return GOSSIP_STOCK_AVAILABLE;
-	}
-
-	return GOSSIP_STOCK_AVAILABLE;
-}
 
 const gchar *
 gossip_utils_show_to_string (GossipShow show)
@@ -767,15 +745,31 @@ gossip_utils_get_pixbuf_from_show (GossipShow show)
 }
 
 const gchar *
-gossip_utils_get_default_status (GossipShow show)
+gossip_utils_get_default_status_show (GossipShow show)
 {
 	switch (show) {
-	case GOSSIP_SHOW_AVAILABLE:
+        case GOSSIP_SHOW_AVAILABLE:
+                return _(AVAILABLE_MESSAGE);
+        case GOSSIP_SHOW_BUSY:
+                return _(BUSY_MESSAGE);
+        case GOSSIP_SHOW_AWAY:
+        case GOSSIP_SHOW_EXT_AWAY:
+                return _(AWAY_MESSAGE);
+        };
+ 
+        return _(AVAILABLE_MESSAGE);
+}
+
+const gchar *
+gossip_utils_get_default_status (GossipPresenceType type)
+{
+	switch (type) {
+	case GOSSIP_PRESENCE_TYPE_AVAILABLE:
 		return _(AVAILABLE_MESSAGE);
-	case GOSSIP_SHOW_BUSY:
+	case GOSSIP_PRESENCE_TYPE_BUSY:
 		return _(BUSY_MESSAGE);
-	case GOSSIP_SHOW_AWAY:
-	case GOSSIP_SHOW_EXT_AWAY:
+	case GOSSIP_PRESENCE_TYPE_AWAY:
+	case GOSSIP_PRESENCE_TYPE_EXT_AWAY:
 		return _(AWAY_MESSAGE);
 	};
 
@@ -919,3 +913,81 @@ gossip_utils_url_regex_match (const gchar *msg,
 		
 	return num_matches;
 }
+
+const gchar *
+gossip_utils_get_stock_from_presence (GossipPresence *presence)
+{
+        const gchar *stock = NULL;
+         
+        switch (gossip_presence_get_type (presence)) {
+        case GOSSIP_PRESENCE_TYPE_AVAILABLE:
+                stock = GOSSIP_STOCK_AVAILABLE;
+                break;
+        case GOSSIP_PRESENCE_TYPE_BUSY:
+                stock = GOSSIP_STOCK_BUSY;
+                break;
+        case GOSSIP_PRESENCE_TYPE_AWAY:
+                stock = GOSSIP_STOCK_AWAY;
+                break;
+        case GOSSIP_PRESENCE_TYPE_EXT_AWAY:
+                stock = GOSSIP_STOCK_EXT_AWAY;
+                break;
+        }
+ 
+        return stock;
+}
+ 
+GdkPixbuf *
+gossip_utils_get_pixbuf_from_presence (GossipPresence *presence)
+{
+        const gchar *stock;
+ 
+        stock = gossip_utils_get_stock_from_presence (presence);
+        return gossip_utils_get_pixbuf_from_stock (stock);
+}
+
+GossipPresenceType
+gossip_utils_get_presence_type_from_show_string (const gchar *str)
+{
+	if (!str || !str[0]) {
+		return GOSSIP_PRESENCE_TYPE_AVAILABLE;
+	}
+	else if (strcmp (str, "dnd") == 0) {
+		return GOSSIP_PRESENCE_TYPE_BUSY;
+	}
+	else if (strcmp (str, "away") == 0) {
+		return GOSSIP_PRESENCE_TYPE_AWAY;
+	}
+	else if (strcmp (str, "xa") == 0) {
+		return GOSSIP_PRESENCE_TYPE_EXT_AWAY;
+	}
+	/* We don't support chat, so treat it like available. */
+	else if (strcmp (str, "chat") == 0) {
+		return GOSSIP_PRESENCE_TYPE_AVAILABLE;
+	}
+
+	return GOSSIP_PRESENCE_TYPE_AVAILABLE;
+}
+
+gint
+gossip_utils_str_case_cmp (const gchar *s1, const gchar *s2)
+{
+	return gossip_utils_str_n_case_cmp (s1, s2, -1);
+}	
+
+gint
+gossip_utils_str_n_case_cmp (const gchar *s1, const gchar *s2, gsize n)
+{
+	gchar *u1, *u2;
+	gint   ret_val;
+	
+	u1 = g_utf8_casefold (s1, n);
+	u2 = g_utf8_casefold (s2, n);
+
+	ret_val = g_utf8_collate (u1, u2);
+	g_free (u1);
+	g_free (u2);
+
+	return ret_val;
+}
+
